@@ -229,12 +229,4 @@ ldns_status ldns_pkt_verify(ldns_pkt *p, ldns_rr_type t, ldns_rdf *o, ldns_rr_li
  */
 ldns_zone *ldns_zone_sign(const ldns_zone *zone, ldns_key_list *key_list);
  
-/**
- * Initialize the random function. This calls OpenSSL
- * \param[in] fd a file providing entropy data
- * \param[in] bytes number of bytes for the seed
- * \return LDNS_STATUS_OK if init succeeds
- */
-ldns_status ldns_init_random(FILE *fd, uint16_t bytes);
-
 #endif /* LDNS_DNSSEC_H */
