@@ -67,7 +67,7 @@ ldns_send_buffer(ldns_pkt **result, ldns_resolver *r, ldns_buffer *qb, ldns_rdf
 	uint8_t *reply_bytes = NULL;
 	size_t reply_size = 0;
 	ldns_status status, send_status;
-
+	
 	assert(r != NULL);
 
 	status = LDNS_STATUS_OK;
@@ -91,6 +91,11 @@ ldns_send_buffer(ldns_pkt **result, ldns_resolver *r, ldns_buffer *qb, ldns_rdf
 		}
 		all_servers_rtt_inf = false;
 
+		/* maybe verbosity setting?
+		printf("Sending to ");
+		ldns_rdf_print(stdout, ns_array[i]);
+		printf("\n");
+		*/
 		ns = ldns_rdf2native_sockaddr_storage(ns_array[i],
 				ldns_resolver_port(r), &ns_len);
 		
@@ -193,8 +198,9 @@ ldns_send_buffer(ldns_pkt **result, ldns_resolver *r, ldns_buffer *qb, ldns_rdf
 			status = LDNS_STATUS_CRYPTO_TSIG_BOGUS;
 		}
 	}
+#else
+	(void)tsig_mac;
 #endif /* HAVE_SSL */
-	
 	LDNS_FREE(reply_bytes);
 	if (result) {
 		*result = reply;
